# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 14:12:41 2014

@author: akusok
"""

from unittest import TestCase
import numpy as np

import hpelm


# noinspection PyArgumentList
class TestAcceptance(TestCase):

    def test_basic_elm_single_machine(self):
        """Just run an ELM with sine function and check training MSE.
        """
        n = 1000
        err = 0.2
        Y = np.linspace(-1, 1, num=n)
        X = np.sin(16 * Y) * Y + np.random.randn(n) * err

        elm = hpelm.ELM(1, 1)
        elm.add_neurons(3, "sigm")
        print elm.nnet.get_B()
        elm.train(X, Y)
        print elm.nnet.get_B()
        Yt = elm.predict(X)

        MSE = np.mean((Y - Yt) ** 2)
        self.assertLess(MSE, 0.5)

    def test_xor_one_neuron_solved(self):
        """ELM should be able to solve XOR problem.
        """
        X = np.array([[0, 0],
                      [1, 1],
                      [1, 0],
                      [0, 1]])
        Y = np.array([1, 1, -1, -1])
        for _ in range(100):  # try 100 random initializations to solve XOR problem
            try:
                elm = hpelm.ELM(2, 1)
                elm.add_neurons(1, "sigm")
                elm.train(X, Y)
                Yh = elm.predict(X)
                self.assertGreater(Yh[0], 0)
                self.assertGreater(Yh[1], 0)
                self.assertLess(Y[2], 0)
                self.assertLess(Y[3], 0)
                nn = sum([n[0] for n in elm.nnet.neurons])
                self.assertEqual(nn, 1)  # one neuron in the ELM
                return
            except AssertionError:
                pass
        self.fail("Cannot train 1 neuron to solve XOR problem in 100 re-initializations")



# -*- coding: utf-8 -*-
"""
Created on Sat Feb 21 20:46:31 2015

@author: akusok
"""

from mrsr import mrsr
from mrsr2 import mrsr2
from hdf5_tools import make_hdf5, normalize_hdf5, _ireader, _iwriter, _prepare_fHH, _write_fHH
from rbf_param import rbf_param


# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 17:48:33 2014

@author: akusok
"""

import numpy as np
import multiprocessing as mp
from time import time
from hpelm.modules import make_hdf5, _ireader, _iwriter, _prepare_fHH, _write_fHH
from tables import open_file
from elm import ELM


class HPELM(ELM):
    """Interface for training High-Performance Extreme Learning Machines (HP-ELM).

    Args:
        inputs (int): dimensionality of input data, or number of data features
        outputs (int): dimensionality of output data, or number of classes
        classification ('c'/'wc'/'ml', optional): train ELM for classfication ('c') / weighted classification ('wc') /
            multi-label classification ('ml'). For weighted classification you can provide weights in `w`. ELM will
            compute and use the corresponding classification error instead of Mean Squared Error.
        w (vector, optional): weights vector for weighted classification, lenght (`outputs` * 1).
        batch (int, optional): batch size for data processing in ELM, reduces memory requirements. Does not work
            for model structure selection (validation, cross-validation, Leave-One-Out). Can be changed later directly
            as a class attribute.
        accelerator (string, optional): type of accelerated ELM to use: None, 'GPU', ...
        precision (optional): data precision to use, supports signle ('single', '32' or numpy.float32) or double
            ('double', '64' or numpy.float64). Single precision is faster but may cause numerical errors. Majority
            of GPUs work in single precision. Default: **double**.
        norm (double, optinal): L2-normalization parameter, **None** gives the default value.
        tprint (int, optional): ELM reports its progess every `tprint` seconds or after every batch,
            whatever takes longer.

    Class attributes; attributes that simply store initialization or `train()` parameters are omitted.

    Attributes:
        nnet (object): Implementation of neural network with computational methods, but without
            complex logic. Different implementations are given by different classes: for Python, for GPU, etc.
            See ``hpelm.nnets`` folder for particular files. You can implement your own computational algorithm
            by inheriting from ``hpelm.nnets.SLFN`` and overwriting some methods.
        flist (list of strings): Awailable types of neurons, use them when adding new neurons.

    Note:
        The 'hdf5' type denotes a name of HDF5 file type with a single 2-dimensional array inside. HPELM uses PyTables
        interface to HDF5: http://www.pytables.org/. For HDF5 array examples, see
        http://www.pytables.org/usersguide/libref/homogenous_storage.html. Array name is irrelevant,
        but there must be **only one array per HDF5 file**.

        A 2-dimensional Numpy.ndarray can also be used.
    """

    def train(self, fX, fT, *args, **kwargs):
        """Universal training interface for HP-ELM model.

        Always trains a basic ELM model without model structure selection.
        L2-regularization is available as `norm` parameter at HPELM initialization.
        Number of neurons selection with validation set for trained HPELM is available in `train_hpv()` method.

        Args:
            fX (hdf5): input data on disk, size (N * `inputs`)
            fT (hdf5): outputs data on disk, size (N * `outputs`)
            'c'/'wc'/'ml' (string, choose one): train HPELM for classification ('c'), classification with weighted
                classes ('wc') or multi-label classification ('ml') with several correct classes per data sample.
                In classification, number of `outputs` is the number of classes; correct class(es) for each sample
                has value 1 and incorrect classes have 0.

        Keyword Args:
            istart (int, optional): index of first data sample to use from `fX`, `istart` < N. If not given,
                all data from `fX` is used. Sample with index `istart` is used for training, indexing is 0-based.
            icount (int, optional): number of data samples to use from `fX`, starting from `istart`, automatically
                adjusted to `istart` + `icount` <= N. If not given, all data starting from `start` is used.
                The last sample used for training is `istart`+`icount`-1, so you can index data as:
                istart_1=0, icount_1=1000; istart_2=1000, icount_2=1000; istart_3=2000, icount_3=1000, ...
            batch (int, optional): batch size for ELM, overwrites batch size from the initialization
        """
        # TODO: move to h5py, because I don't need pyTables features
        # TODO: move to h5py with MPI async IO (driver='mpio')
        # TODO: explain why I don't support parallel processing (huge amount of data to transfer, or fast enough)
        X, T = self._checkdata(fX, fT)
        self._train_parse_args(args, kwargs)

        istart = 0
        icount = np.inf
        if "istart" in kwargs.keys():
            istart = max(0, int(kwargs["istart"]))
        if "icount" in kwargs.keys():
            icount = kwargs["icount"]
        self.add_data(X, T, istart=istart, icount=icount)
        self.nnet.solve()

    def add_data(self, fX, fT, istart=0, icount=np.inf, fHH=None, fHT=None):
        """Feed new training data (X,T) to HP-ELM model in batches: does not solve ELM itself.

        This method prepares an intermediate solution data, that takes the most time. After that, obtaining
        the solution is fast.

        The intermediate solution consists of two matrices: `HH` and `HT`. They can be in memory for a model computed
        at once, or stored on disk for a model computed in parts or in parallel.

        For iterative solution, provide file names for on-disk matrices in the input parameters `fHH` and `fHT`.
        They will be created if they don't exist, or new results will be merged with the existing ones. This method is
        multiprocess-safe for parallel writing into files `fHH` and `fHT`, that allows you to easily compute ELM
        in parallel. The multiprocess-safeness uses Python module 'fasteners' and a lock file, which is named
        fHH+'.lock' and fHT+'.lock'.

        Args:
            fX (hdf5): (part of) input training data size (N * `inputs`)
            fT (hdf5) (part of) output training data size (N * `outputs`)
            istart (int, optional): index of first data sample to use from `fX`, `istart` < N. If not given,
                all data from `fX` is used. Sample with index `istart` is used for training, indexing is 0-based.
            icount (int, optional): number of data samples to use from `fX`, starting from `istart`, automatically
                adjusted to `istart` + `icount` <= N. If not given, all data starting from `start` is used.
                The last sample used for training is `istart`+`icount`-1, so you can index data as:
                istart_1=0, icount_1=1000; istart_2=1000, icount_2=1000; istart_3=2000, icount_3=1000, ...
            fHH, fHT (string, optional): file names for storing HH and HT matrices. Files are created if they don't
                exist, or new result is added to the existing files if they exist. Parallel writing to the same
                `fHH`, `fHT` files is multiprocess-safe, made specially for parallel training of HP-ELM. Another use
                is to split a very long training of huge ELM into smaller parts, so the training can be interrupted
                and resumed later.

        """
        # initialize
        assert len(self.nnet.neurons) > 0, "Add neurons to ELM before using it"
        X, T = self._checkdata(fX, fT)
        N = X.shape[0]
        _prepare_fHH(fHH, fHT, self.nnet, self.precision)
        # custom range adjustments
        icount = min(istart + icount, N)
        nb = int(np.ceil(float(icount) / self.batch))  # number of batches

        # weighted classification initialization
        if self.classification == "wc" and self.wc is None:
            ns = np.zeros((self.nnet.outputs,))
            for b in xrange(nb):  # batch sum is much faster
                start = b*self.batch + istart
                stop = min((b+1)*self.batch + istart, icount + istart)
                ns += T[start:stop].sum(axis=0)
            ns = ns.astype(self.precision)
            self.wc = ns.sum() / ns  # class weights normalized to number of samples

        # main loop over all the data
        t = time()
        t0 = time()
        eta = 0
        wc_vector = None
        for b in xrange(nb):
            start = b*self.batch + istart
            stop = min((b+1)*self.batch + istart, icount + istart)
            Xb = X[start:stop]
            Tb = T[start:stop]
            if self.classification == "wc":
                wc_vector = self.wc[np.where(Tb == 1)[1]]  # weights for samples in the batch

            self.nnet.add_batch(Xb, Tb, wc_vector)

            # report time
            eta = int(((time()-t0) / (b+1)) * (nb-b-1))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()

        # if storing output to disk
        if fHH is not None and fHT is not None:
            HH, HT = self.nnet.get_corr()
            HH[np.diag_indices_from(HH)] -= self.nnet.norm  # norm is already included
            _write_fHH(fHH, fHT, HH, HT)

    def solve_corr(self, fHH, fHT):
        """Solves an ELM model with the given (covariance) fHH and (correlation) fHT HDF5 files.

        Args:
            fHH (hdf5): an hdf5 file with intermediate solution data
            fHT (hdf5): an hdf5 file with intermediate solution data
        """
        HH, HT = self._checkcorr(fHH, fHT)
        B = self.nnet.solve_corr(HH, HT)
        self.nnet.set_B(B)

    def predict(self, fX, fY, istart=0, icount=np.inf):
        """Iterative predict outputs and save them to HDF5, can use custom range.

        Args:
            fX (hdf5): hdf5 filename with input data from which outputs are predicted
            fY (hdf5): hdf5 filename to store output data into
            istart (int, optional): index of first data sample to use from `fX`, `istart` < N. If not given,
                all data from `fX` is used. Sample with index `istart` is used for training, indexing is 0-based.
            icount (int, optional): number of data samples to use from `fX`, starting from `istart`, automatically
                adjusted to `istart` + `icount` <= N. If not given, all data starting from `start` is used.
                The last sample used for training is `istart`+`icount`-1, so you can index data as:
                istart_1=0, icount_1=1000; istart_2=1000, icount_2=1000; istart_3=2000, icount_3=1000, ...
        """
        assert len(self.nnet.neurons) > 0, "Add neurons to ELM and train it before using"
        assert self.nnet.B is not None, "Train ELM before predicting"
        X, _ = self._checkdata(fX, None)
        N = X.shape[0]
        # custom range adjustments
        icount = min(istart + icount, N)
        nb = int(np.ceil(float(icount) / self.batch))  # number of batches
        # make file to store results
        make_hdf5((icount, self.nnet.outputs), fY, dtype=self.precision)
        h5 = open_file(fY, "a")
        for Y in h5.walk_nodes():
            pass  # find a node with whatever name

        t = time()
        t0 = time()
        eta = 0
        for b in xrange(0, nb):
            start = b*self.batch + istart
            stop = min((b+1)*self.batch + istart, icount + istart)

            # get data
            Xb = X[start:stop]
            # process data
            Yb = self.nnet._predict(Xb)
            # write data
            Y[start-istart:stop-istart] = Yb

            # report time
            eta = int(((time()-t0) / (b+1)) * (nb-b-1))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()

        h5.flush()
        h5.close()

    def project(self, fX, fH, istart=0, icount=np.inf):
        """Iteratively project input data from HDF5 into HPELM hidden layer, and save in another HDF5.

        Args:
            fX (hdf5): hdf5 filename with input data from which outputs are predicted
            fH (hdf5): hdf5 filename to store output data into
            istart (int, optional): index of first data sample to use from `fX`, `istart` < N. If not given,
                all data from `fX` is used. Sample with index `istart` is used for training, indexing is 0-based.
            icount (int, optional): number of data samples to use from `fX`, starting from `istart`, automatically
                adjusted to `istart` + `icount` <= N. If not given, all data starting from `start` is used.
                The last sample used for training is `istart`+`icount`-1, so you can index data as:
                istart_1=0, icount_1=1000; istart_2=1000, icount_2=1000; istart_3=2000, icount_3=1000, ...
        """
        assert len(self.nnet.neurons) > 0, "Add neurons to ELM before using it"
        X, _ = self._checkdata(fX, None)
        N = X.shape[0]
        # custom range adjustments
        icount = min(istart + icount, N)
        nb = int(np.ceil(float(icount) / self.batch))  # number of batches
        # make file to store results
        make_hdf5((icount, self.nnet.L), fH, dtype=self.precision)
        h5 = open_file(fH, "a")
        for H in h5.walk_nodes():
            pass  # find a node with whatever name

        t = time()
        t0 = time()
        for b in xrange(0, nb):
            start = b*self.batch + istart
            stop = min((b+1)*self.batch + istart, icount + istart)

            # get data
            Xb = X[start:stop]
            # process data
            Hb = self.nnet._project(Xb)
            # write data
            H[start-start:stop-istart] = Hb

            # report time
            eta = int(((time()-t0) / (b+1)) * (nb-b-1))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()

        h5.flush()
        h5.close()

    def error(self, fT, fY, istart=0, icount=np.inf):
        """Calculate error of model predictions of HPELM.

        Computes Mean Squared Error (MSE) between model predictions Y and true outputs T.
        For classification, computes mis-classification error.
        For multi-label classification, correct classes are all with Y>0.5.

        For weighted classification the error is an average weighted True Positive Rate,
        or percentage of correctly predicted samples for each class, multiplied by weight
        of that class and averaged. If you want something else, just write it yourself :)
        See https://en.wikipedia.org/wiki/Confusion_matrix for details.

        Args:
            fT (hdf5): hdf5 filename with true outputs
            fY (hdf5): hdf5 filename with predicted outputs
            istart (int, optional): index of first data sample to use from `fX`, `istart` < N. If not given,
                all data from `fX` is used. Sample with index `istart` is used for training, indexing is 0-based.
            icount (int, optional): number of data samples to use from `fX`, starting from `istart`, automatically
                adjusted to `istart` + `icount` <= N. If not given, all data starting from `start` is used.
                The last sample used for training is `istart`+`icount`-1, so you can index data as:
                istart_1=0, icount_1=1000; istart_2=1000, icount_2=1000; istart_3=2000, icount_3=1000, ...

        Returns:
            e (double): MSE for regression / classification error for classification.
        """
        _, T = self._checkdata(None, fT)
        _, Y = self._checkdata(None, fY)
        return self._error(T, Y, istart=istart, icount=icount)

    def _error(self, T, Y, istart=0, icount=np.inf):
        """Iterative batch error calcualtion.

        Args:
            T (matrix): true outputs for error calculation
            Y (matrix): predicted outputs for error calculation
            istart (int): index of first sample to process
            icount (int): number of samples to process
        """
        N = T.shape[0]
        icount = min(istart + icount, N)
        nb = int(np.ceil(float(icount) / self.batch))  # number of batches

        if self.classification == "c":
            err = 0
            for b in xrange(nb):
                start = b*self.batch + istart
                stop = min((b+1)*self.batch + istart, icount + istart)
                Tb = np.array(T[start:stop])
                Yb = np.array(Y[start:stop])
                errb = np.mean(Yb.argmax(1) != Tb.argmax(1))
                err += errb * float(stop-start)/icount

        elif self.classification == "wc":  # weighted classification
            c = T.shape[1]
            errc = np.zeros(c)
            countc = np.zeros(c)
            for b in xrange(nb):
                start = b*self.batch + istart
                stop = min((b+1)*self.batch + istart, icount + istart)
                Tb = np.array(T[start:stop])
                Yb = np.array(Y[start:stop])
                for i in xrange(c):  # per-class MSE
                    idx = np.where(Tb[:, i] == 1)[0]
                    if len(idx) > 0:
                        err1 = np.not_equal(Yb[idx].argmax(1), i)
                        errc[i] += err1.sum()
                        countc[i] += len(idx)
            errc = errc / countc  # get mean value
            err = np.sum(errc * self.wc) / np.sum(self.wc)

        elif self.classification == "ml":
            err = 0
            for b in xrange(nb):
                start = b*self.batch + istart
                stop = min((b+1)*self.batch + istart, icount + istart)
                Tb = np.array(T[start:stop])
                Yb = np.array(Y[start:stop])
                errb = np.not_equal(Yb > 0.5, Tb > 0.5).mean()
                err += errb * float(stop-start)/icount

        else:  # MSE error
            err = 0
            for b in xrange(nb):
                start = b*self.batch + istart
                stop = min((b+1)*self.batch + istart, icount + istart)
                Tb = T[start:stop]
                Yb = Y[start:stop]
                errb = np.mean((Tb - Yb)**2)
                err += errb * float(stop-start)/icount

        return err

    def validation_corr(self, fHH, fHT, fXv, fTv, steps=10):
        """Quick batch error evaluation with different numbers of neurons on a validation set.

        Only feasible implementation of model structure selection with HP-ELM. This method makes a single pass
        over the validation data, computing errors for all numbers of neurons at once. It requires HDF5 files with
        matrices HH and HT: `fHH` and `fHT`, obtained from `add_data(..., fHH, fHT)` method.

        The method writes the best solution to the HPELM model.

        Args:
            fHH (string): name of HDF5 file with HH matrix
            fHT (string): name of HDF5 file with HT matrix
            fXv (string): name of HDF5 file with validation dataset inputs
            fTv (string): name of HDF5 file with validation dataset outputs
            steps (int or vector): amount of different numbers of neurons to test, choosen uniformly on a logarithmic
                scale from 3 to number of neurons in HPELM. Can be given exactly as a vector.

        Returns:
            Ls (vector): numbers of neurons used by `validation_corr()` method
            errs (vector): corresponding errors for number of neurons in `Ls`, with classification error if model
                is run for classification
            confs (list of matrix): list of confusion matrices corresponding to elements in Ls (empty for regression)
        """
        X, T = self._checkdata(fXv, fTv)
        HH, HT = self._checkcorr(fHH, fHT)
        N = X.shape[0]
        L = self.nnet.L
        classification = self.classification is not None

        Ls = np.logspace(np.log(3), np.log(L), steps, base=np.e, endpoint=True)
        Ls = np.ceil(Ls).astype(np.int)
        Ls = np.unique(Ls)  # numbers of neurons to check
        k = Ls.shape[0]
        errs = np.zeros((k,))  # errors for these numbers of neurons
        nb = int(np.ceil(float(N) / self.batch))

        Betas = []  # keep all betas in memory
        confs = []
        for l in Ls:
            Betas.append(self.nnet.solve_corr(HH[:l, :l], HT[:l, :]))
            if classification: confs.append(np.zeros((self.nnet.outputs, self.nnet.outputs)))

        t = time()
        t0 = time()
        for b in xrange(nb):
            start = b*self.batch
            stop = min((b+1)*self.batch, N)
            Tb = np.array(T[start:stop])
            Xb = np.array(X[start:stop])
            Hb = self.nnet._project(Xb)
            for i in xrange(k):
                hb1 = Hb[:, :Ls[i]]
                Yb = np.dot(hb1, Betas[i])
                errs[i] += self._error(Tb, Yb) * float(stop-start)/N
                if classification: confs[i] += self.confusion(Tb, Yb)
            # report time
            eta = int(((time()-t0) / (b+1)) * (nb-b-1))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()

        k_opt = np.argmin(errs)
        best_L = Ls[k_opt]
        self.nnet._prune(np.arange(best_L))
        self.nnet.set_B(Betas[k_opt])
        del Betas
        print "%d of %d neurons selected with a validation set" % (best_L, L)
        if best_L > L*0.9:
            print "Hint: try re-training with more hidden neurons"
        return Ls, errs, confs

    # async-IO versions of methods

    def train_async(self, fX, fT, *args, **kwargs):
        """Training HPELM with asyncronous I/O, good for network drives, etc. See `train()` for reference.

        Spawns new processes using Python's `multiprocessing` module.
        """
        X, T = self._checkdata(fX, fT)
        self._train_parse_args(args, kwargs)

        istart = 0
        icount = np.inf
        if "istart" in kwargs.keys():
            istart = max(0, int(kwargs["istart"]))
        if "icount" in kwargs.keys():
            icount = kwargs["icount"]
        self.add_data_async(fX, fT, istart=istart, icount=icount)
        self.nnet.solve()

    def add_data_async(self, fX, fT, istart=0, icount=np.inf, fHH=None, fHT=None):
        """Version of `add_data()` with asyncronous I/O. See `add_data()` for reference.

        Spawns new processes using Python's `multiprocessing` module, and requires more memory than non-async version.
        """
        # initialize
        assert len(self.nnet.neurons) > 0, "Add neurons to ELM before using it"
        X, T = self._checkdata(fX, fT)
        N = X.shape[0]
        # TODO: adapt for GPU solver
        _prepare_fHH(fHH, fHT, self.nnet, self.precision)
        # custom range adjustments
        icount = min(istart + icount, N)
        nb = int(np.ceil(float(icount) / self.batch))

        # weighted classification initialization
        if self.classification == "wc" and self.wc is None:
            ns = np.zeros((self.nnet.outputs,))
            for b in xrange(nb):  # batch sum is much faster
                start = b*self.batch + istart
                stop = min((b+1)*self.batch + istart, icount + istart)
                ns += T[start:stop].sum(axis=0)
            ns = ns.astype(self.precision)
            self.wc = ns.sum() / ns  # class weights normalized to number of samples

        # close X and T files opened by _checkdata()
        h5 = self.opened_hdf5.pop()
        h5.close()
        h5 = self.opened_hdf5.pop()
        h5.close()

        # start async reader and writer for HDF5 files
        qX_in = mp.Queue()
        qX_out = mp.Queue(1)
        readerX = mp.Process(target=_ireader, args=(fX, qX_in, qX_out))
        readerX.daemon = True
        readerX.start()
        qT_in = mp.Queue()
        qT_out = mp.Queue(1)
        readerT = mp.Process(target=_ireader, args=(fT, qT_in, qT_out))
        readerT.daemon = True
        readerT.start()

        # main loop over all the data
        t = time()
        t0 = time()
        eta = 0
        wc_vector = None
        for b in xrange(0, nb+1):
            start_next = b*self.batch + istart
            stop_next = min((b+1)*self.batch + istart, icount + istart)
            # prefetch data
            qX_in.put((start_next, stop_next))  # asyncronous reading of next data batch
            qT_in.put((start_next, stop_next))

            if b > 0:  # first iteration only prefetches data
                Xb = qX_out.get()
                Tb = qT_out.get()
                if self.classification == "wc":
                    wc_vector = self.wc[np.where(Tb == 1)[1]]  # weights for samples in the batch

                self.nnet.add_batch(Xb, Tb, wc_vector)

            # report time
            eta = int(((time()-t0) / (b+1)) * (nb-b-1))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()

        # close async reader and writer
        readerX.join()
        readerT.join()

        # if storing output to disk
        if fHH is not None and fHT is not None:
            HH, HT = self.nnet.get_corr()
            HH[np.diag_indices_from(HH)] -= self.nnet.norm  # norm is already included
            _write_fHH(fHH, fHT, HH, HT)

    def predict_async(self, fX, fY, istart=0, icount=np.inf):
        """Version of `predict()` with asyncronous I/O. See `predict()` for reference.

        Spawns new processes using Python's `multiprocessing` module, and requires more memory than non-async version.
        """
        assert len(self.nnet.neurons) > 0, "Add neurons to ELM and train it before using"
        assert self.nnet.B is not None, "Train ELM before predicting"
        X, _ = self._checkdata(fX, None)
        N = X.shape[0]
        # custom range adjustments
        icount = min(istart + icount, N)
        nb = int(np.ceil(float(icount) / self.batch))  # number of batches
        # make file to store results
        make_hdf5((icount, self.nnet.outputs), fY)

        # start async reader and writer for HDF5 files
        qr_in = mp.Queue()
        qr_out = mp.Queue(1)
        reader = mp.Process(target=_ireader, args=(fX, qr_in, qr_out))
        reader.daemon = True
        reader.start()
        qw_in = mp.Queue(1)
        writer = mp.Process(target=_iwriter, args=(fY, qw_in))
        writer.daemon = True
        writer.start()

        t = time()
        t0 = time()
        eta = 0
        for b in xrange(0, nb+1):
            start_next = b*self.batch + istart
            stop_next = min((b+1)*self.batch + istart, icount + istart)
            # prefetch data
            qr_in.put((start_next, stop_next))  # asyncronous reading of next data batch

            if b > 0:  # first iteration only prefetches data
                # get data
                Xb = qr_out.get()
                # process data
                Yb = self.nnet._predict(Xb)
                # save data
                qw_in.put((Yb, start-istart, stop-istart))

            start = start_next
            stop = stop_next
            # report time
            eta = int(((time()-t0) / (b+1)) * (nb-b-1))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()

        qw_in.put(None)
        reader.join()
        writer.join()

    def _checkcorr(self, fHH, fHT):
        """Analog of `_checkdata()` for correlation matrices.
        """
        try:
            h5 = open_file(fHH, "r")
        except:
            raise IOError("Cannot read HDF5 file at %s" % fHH)
        node = None
        for node in h5.walk_nodes():
            pass  # find a node with whatever name
        if node:
            HH = node[:]
        else:
            raise IOError("Empty HDF5 file at %s" % fHH)
        h5.close()

        try:
            h5 = open_file(fHT, "r")
        except:
            raise IOError("Cannot read HDF5 file at %s" % fHT)
        node = None
        for node in h5.walk_nodes():
            pass  # find a node with whatever name
        if node:
            HT = node[:]
        else:
            raise IOError("Empty HDF5 file at %s" % fHT)
        h5.close()

        L = self.nnet.L
        c = self.nnet.outputs
        assert len(self.nnet.neurons) > 0, "Cannot solve ELM without neurons"
        assert HH.shape[0] == L and HH.shape[1] == L, "HH has wrong shape: (%d,%d) expected, (%d,%d) found" \
                                                      % (L, L, HH.shape[0], HH.shape[1])
        assert HT.shape[0] == L and HT.shape[1] == c, "HT has wrong shape: (%d,%d) expected, (%d,%d) found" \
                                                      % (L, c, HH.shape[0], HH.shape[1])
        return HH, HT


# -*- coding: utf-8 -*-


import numpy as np
from scipy import optimize
from scipy.linalg import lu_factor, lu_solve


def mrsr2(X, T, kmax, norm=2):
    """Multi-Responce Sparse Regression implementation with linear scaling in number of outputs.

    Basically an L1-regularized regression with multiple outputs, regularization considers all outputs together,
    method returns the best input features one by one and can be stopped early. Compared to an original MRSR this method
    is slower for small problems, but has a linear complexity in the number of outputs instead of exponential one,
    so it is suitable for auto-encoders and other tasks with large output dimensionality.

    Args:
        T (matrix): an (n x p) matrix of targets. The columns of T should have zero mean and same scale (e.g. equal variance).
        X (matrix): an (n x m) matrix of regressors. The columns of X should have zero mean and same scale (e.g. equal variance).
        kmax (int): an integer fixing the number of steps to be run, which equals to the maximum number of regressors in the model.
        norm (from Numpy.linalg.norm): norm to use in MRSR2, can be `1` for L1 or `2` for L2 norm, default `2`.

    Returns:
        i1 (vector): a (1 x kmax) vector of indices revealing the order in which the regressors enter model.

    Reference:
        Better MRSR implementation according to:
        "Common subset selection of inputs in multiresponse regression" by
        Timo Similä and Jarkko Tikka, International Joint Conference on Neural Networks 2006

    Created on Sun Jan 26 13:48:54 2014
    @author: Anton Akusok
    """

    # initializing
    ins = X.shape[1]
    outs = T.shape[1]
    X = np.array(X, order='F')  # Fortran ordering is good for operating columns 
    XA = np.empty(X.shape, order='F')
    XX = np.dot(X.T, X)        
    XT = np.dot(X.T, T)
    rank = []  # active inputs list
    nonrank = range(ins)
    W = np.zeros((ins, outs))  # current projection estimator
    Y = np.zeros(T.shape)  # current target estimator
    Yk1 = np.zeros(T.shape)
    Wk1 = np.zeros((1, outs))
    j_current = None  # currently added input dimension
    kmax = min(kmax, X.shape[0])    
    
    # get ranking
    for _ in xrange(kmax):
        
        # first step
        if len(rank) == 0:
            c_max = -1
            for j in nonrank:
                c_kj = np.linalg.norm(np.dot(T.T, X[:,j]), norm)
                if c_kj > c_max:
                    c_max = c_kj
                    j_max = j
            j_current = j_max  
            # save new input
            rank.append(j_current)
            nonrank.remove(j_current)
            # swap columns
            idx = len(rank)-1
            XA[:,idx] = X[:, j_current]
            y_min = 1
        
        # last step
        elif len(nonrank) == 0:  
            Y = Yk1
            W[rank] = Wk1
            y_min = 1

        # intermediate step        
        else:  
            Yk2 = (Yk1 - Y).T
            T2 = (T - Y).T
            
            X9 = X[:, j_current]
            c_max = np.linalg.norm(np.dot(T2, X9), norm)
            #c_max = np.linalg.norm(np.dot(T2, X[:, j_current]), norm)
            
            #fun = lambda y,x_new: (1-y)*c_max - np.linalg.norm(T2.dot(x_new) - y*Yk2.dot(x_new))
            fun_p = lambda y,p1,p2: (1-y)*c_max - np.linalg.norm(p1 - y*p2)  # super fast parametrized function
    
            # find optimal step (minimum over possible additional inputs) 
            y_min = 1  # upper interval
            for j_new in nonrank:
                x_new = X[:,j_new]
                # pre-calculate constant parts of the optimization function for the given x_new
                p1 = T2.dot(x_new)
                p2 = Yk2.dot(x_new)
                if (1-y_min)*c_max < np.linalg.norm(p1 - y_min*p2):  # skip optimization if min(fun) > y_min
                    try:
                        zero = 1E-15  # finding a value greater than zero
                        y_kj = optimize.brentq(fun_p, zero, y_min, xtol=1E-6, args=(p1,p2))
                        y_min = y_kj
                        j_min = j_new
                    except ValueError:  
                        # ValueError: f(a) and f(b) must have different signs
                        # here f(a) < 0 and f(b) < 0; does not fit our purposes anyway 
                        # ignoring this case
                        pass                  
                    
            if y_min == 1:  # if no suitable solution was found
                j_min = j_new
            j_current = j_min

            # add new input into model
            rank.append(j_current)
            nonrank.remove(j_current)
            # add new input to X matrix
            idx = len(rank)-1
            XA[:,idx] = X[:,j_current]

        # post-update ELM estimation with current set of inputs, with LU-ELM
        XtX = XX[rank,:][:,rank]
        XtT = XT[rank,:]
        LU, piv = lu_factor(XtX)#, overwrite_a=True)
        Wk1 = lu_solve((LU, piv), XtT)#, overwrite_b=True)
        X1 = XA[:,:len(rank)]  # replace fancy indexing with simple one
        Yk1 = np.dot(X1, Wk1)  

        if len(rank) > 1:
            # perform variable length step
            Y = (1-y_min)*Y + y_min*Yk1
            W = (1-y_min)*W     
            W[rank] += y_min*Wk1   
               
    # done, return ranking
    return rank

        


























































# -*- coding: utf-8 -*-
"""Copy of test_correctness.py
Created on Wed Sep 23 21:15:18 2015

@author: akusok
"""


from unittest import TestCase
import numpy as np
import tempfile
import os

from hpelm import HPELM
from hpelm.modules.hdf5_tools import make_hdf5


# noinspection PyArgumentList
class TestCorrectness(TestCase):
    tfiles = None

    def makeh5(self, data):
        f, fname = tempfile.mkstemp()
        os.close(f)
        self.tfiles.append(fname)
        make_hdf5(data, fname)
        return fname

    def makefile(self):
        f, fname = tempfile.mkstemp()
        os.close(f)
        os.remove(fname)
        self.tfiles.append(fname)
        return fname

    def setUp(self):
        self.tfiles = []

    def tearDown(self):
        for fname in self.tfiles:
            os.remove(fname)

    def test_NonNumpyInputs_RaiseError(self):
        X = np.array([['1', '2'], ['3', '4'], ['5', '6']])
        T = self.makeh5(np.array([[1], [2], [3]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, hpelm.train, X, T)

    def test_NonNumpyTargets_RaiseError(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = np.array([['a'], ['b'], ['c']])
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, hpelm.train, X, T)

    def test_OneDimensionInputs_RunsCorrectly(self):
        X = self.makeh5(np.array([1, 2, 3]))
        T = self.makeh5(np.array([[1], [2], [3]]))
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)

    def test_OneDimensionTargets_RunsCorrectly(self):
        X = self.makeh5(np.array([1, 2, 3]))
        T = self.makeh5(np.array([1, 2, 3]))
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)

    def test_WrongDimensionalityInputs_RaiseError(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[1], [2], [3]]))
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, hpelm.train, X, T)

    def test_WrongDimensionalityTargets_RaiseError(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[1], [2], [3]]))
        hpelm = HPELM(1, 2)
        hpelm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, hpelm.train, X, T)

    def test_ZeroInputs_RunsCorrectly(self):
        X = self.makeh5(np.array([[0, 0], [0, 0], [0, 0]]))
        T = self.makeh5(np.array([1, 2, 3]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)

    def test_OneDimensionTargets2_RunsCorrectly(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[0], [0], [0]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)

    def test_TrainWithoutNeurons_RaiseError(self):
        X = self.makeh5(np.array([1, 2, 3]))
        T = self.makeh5(np.array([1, 2, 3]))
        hpelm = HPELM(1, 1)
        self.assertRaises(AssertionError, hpelm.train, X, T)

    def test_DifferentNumberOfSamples_RaiseError(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[1], [2]]))
        hpelm = HPELM(2, 1)
        self.assertRaises(AssertionError, hpelm.train, X, T)

    def test_LinearNeurons_MoreThanInputs_Truncated(self):
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "lin")
        self.assertEqual(2, hpelm.nnet.get_neurons()[0][0])

    def test_LinearNeurons_DefaultMatrix_Identity(self):
        hpelm = HPELM(4, 1)
        hpelm.add_neurons(3, "lin")
        np.testing.assert_array_almost_equal(np.eye(4, 3), hpelm.nnet.get_neurons()[0][2])

    def test_SLFN_AddLinearNeurons_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "lin")
        self.assertEquals("lin", hpelm.nnet.get_neurons()[0][1])

    def test_SLFN_AddSigmoidalNeurons_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "sigm")
        self.assertEquals("sigm", hpelm.nnet.get_neurons()[0][1])

    def test_SLFN_AddTanhNeurons_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "tanh")
        self.assertEquals("tanh", hpelm.nnet.get_neurons()[0][1])

    def test_SLFN_AddRbfL1Neurons_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "rbf_l1")
        self.assertEquals("rbf_l1", hpelm.nnet.get_neurons()[0][1])

    def test_SLFN_AddRbfL2Neurons_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "rbf_l2")
        self.assertEquals("rbf_l2", hpelm.nnet.get_neurons()[0][1])

    def test_SLFN_AddRbfLinfNeurons_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "rbf_linf")
        self.assertEquals("rbf_linf", hpelm.nnet.get_neurons()[0][1])

    def test_SLFN_AddUfuncNeurons_GotThem(self):
        hpelm = HPELM(1, 1)
        func = np.frompyfunc(lambda a: a+1, 1, 1)
        hpelm.add_neurons(1, func)
        self.assertIs(func, hpelm.nnet.get_neurons()[0][1])

    def test_SLFN_AddTwoNeuronTypes_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.add_neurons(1, "sigm")
        self.assertEquals(2, len(hpelm.nnet.get_neurons()))
        ntypes = [nr[1] for nr in hpelm.nnet.get_neurons()]
        self.assertIn("lin", ntypes)
        self.assertIn("sigm", ntypes)

    def test_SLFN_AddNeuronsTwice_GotThem(self):
        hpelm = HPELM(1, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.add_neurons(1, "lin")
        self.assertEquals(1, len(hpelm.nnet.get_neurons()))
        self.assertEquals(2, hpelm.nnet.get_neurons()[0][0])

    def test_AddNeurons_InitBias_BiasInModel(self):
        hpelm = HPELM(1, 1)
        bias = np.array([1, 2, 3])
        hpelm.add_neurons(3, "sigm", None, bias)
        neurons = hpelm.nnet.get_neurons()
        np.testing.assert_array_almost_equal(bias, neurons[0][3])

    def test_AddNeurons_InitW_WInModel(self):
        hpelm = HPELM(2, 1)
        W = np.array([[1, 2, 3], [4, 5, 6]])
        hpelm.add_neurons(3, "sigm", W, None)
        np.testing.assert_array_almost_equal(W, hpelm.nnet.get_neurons()[0][2])

    def test_AddNeurons_InitDefault_BiasWNotZero(self):
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "sigm")
        W = hpelm.nnet.get_neurons()[0][2]
        bias = hpelm.nnet.get_neurons()[0][3]
        self.assertGreater(np.sum(np.abs(W)), 0.001)
        self.assertGreater(np.sum(np.abs(bias)), 0.001)

    def test_AddNeurons_InitTwiceBiasW_CorrectlyMerged(self):
        hpelm = HPELM(2, 1)
        W1 = np.random.rand(2, 3)
        W2 = np.random.rand(2, 4)
        bias1 = np.random.rand(3,)
        bias2 = np.random.rand(4,)
        hpelm.add_neurons(3, "sigm", W1, bias1)
        hpelm.add_neurons(4, "sigm", W2, bias2)
        np.testing.assert_array_almost_equal(np.hstack((W1, W2)), hpelm.nnet.get_neurons()[0][2])
        np.testing.assert_array_almost_equal(np.hstack((bias1, bias2)), hpelm.nnet.get_neurons()[0][3])

    def test_TrainIstart_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[1], [2], [3]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T, istart=1)

    def test_TrainIcount_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[1], [2], [3]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T, icount=2)

    def test_TrainIstart_HasEffect(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[3], [2], [3]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)
        B1 = hpelm.nnet.get_B()
        hpelm.train(X, T, istart=1)
        B2 = hpelm.nnet.get_B()
        self.assertFalse(np.allclose(B1, B2), "iStart index does not work")

    def test_TrainIcount_HasEffect(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[3], [2], [3]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)
        B1 = hpelm.nnet.get_B()
        hpelm.train(X, T, icount=2)
        B2 = hpelm.nnet.get_B()
        self.assertFalse(np.allclose(B1, B2), "iCount index does not work")

    def test_TrainAsync_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6]]))
        T = self.makeh5(np.array([[1], [2], [3]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train_async(X, T)

    def test_TrainAsyncWeighted_Works(self):
        X = self.makeh5(np.array([1, 2, 3, 1, 2, 3]))
        T = self.makeh5(np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]]))
        hpelm = HPELM(1, 2)
        hpelm.add_neurons(1, "lin")
        hpelm.train_async(X, T, 'wc', wc=(1,2))

    def test_TrainAsyncIndexed_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train_async(X, T, istart=1, icount=2)

    def test_WeightedClassification_Works(self):
        X = self.makeh5(np.array([1, 2, 3, 1, 2, 3]))
        T = self.makeh5(np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]]))
        hpelm = HPELM(1, 2)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T, 'wc', w=(1, 1))

    def test_WeightedClassification_DefaultWeightsWork(self):
        X = self.makeh5(np.array([1, 2, 3, 1, 2, 3]))
        T = self.makeh5(np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]]))
        hpelm = HPELM(1, 2)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T, 'wc')

    def test_HPELM_tprint(self):
        X = self.makeh5(np.array([1, 2, 3, 1, 2, 3]))
        T = self.makeh5(np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]]))
        hpelm = HPELM(1, 2, batch=2, tprint=0)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)

    def test_AddDataToFile_SingleAddition(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "lin")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data(X, T, fHH=fHH, fHT=fHT)

    def test_AddDataToFile_MultipleAdditions(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "lin")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data(X, T, fHH=fHH, fHT=fHT)
        hpelm.add_data(X, T, fHH=fHH, fHT=fHT)

    def test_AddDataAsyncToFile_SingleAddition(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "lin")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data_async(X, T, fHH=fHH, fHT=fHT)

    def test_AddDataAsyncToFile_MultipleAdditions(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "lin")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data_async(X, T, fHH=fHH, fHT=fHT)
        hpelm.add_data_async(X, T, fHH=fHH, fHT=fHT)

    def test_AddDataToFile_MixedSequentialAsync(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "lin")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data(X, T, fHH=fHH, fHT=fHT)
        hpelm.add_data_async(X, T, fHH=fHH, fHT=fHT)

    def test_SolveCorr_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(3, "lin")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data(X, T, fHH=fHH, fHT=fHT)
        hpelm.solve_corr(fHH, fHT)
        self.assertIsNot(hpelm.nnet.get_B(), None)

    def test_ValidationCorr_Works(self):
        X = self.makeh5(np.random.rand(30, 3))
        T = self.makeh5(np.random.rand(30, 2))
        hpelm = HPELM(3, 2, norm=1e-6)
        hpelm.add_neurons(6, "tanh")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data(X, T, fHH=fHH, fHT=fHT)
        nns, err, confs = hpelm.validation_corr(fHH, fHT, X, T, steps=3)
        self.assertGreater(err[0], err[-1])

    def test_ValidationCorr_ReturnsConfusion(self):
        X = self.makeh5(np.random.rand(10, 3))
        T = self.makeh5(np.random.rand(10, 2))
        hpelm = HPELM(3, 2, classification="c")
        hpelm.add_neurons(6, "tanh")
        fHH = self.makefile()
        fHT = self.makefile()
        hpelm.add_data(X, T, fHH=fHH, fHT=fHT)
        _, _, confs = hpelm.validation_corr(fHH, fHT, X, T, steps=3)
        self.assertGreater(np.sum(confs[0]), 1)

    def test_Predict_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)
        fY = self.makefile()
        hpelm.predict(X, fY)

    def test_PredictAsync_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)
        fY = self.makefile()
        hpelm.predict_async(X, fY)

    def test_Project_Works(self):
        X = self.makeh5(np.array([[1, 2], [3, 4], [5, 6], [7, 8]]))
        T = self.makeh5(np.array([[1], [2], [3], [4]]))
        hpelm = HPELM(2, 1)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T)
        fH = self.makefile()
        hpelm.project(X, fH)

    def test_RegressionError_Works(self):
        T = np.array([1, 2, 3])
        Y = np.array([1.1, 2.2, 3.3])
        err1 = np.mean((T - Y) ** 2)
        fT = self.makeh5(T)
        fY = self.makeh5(Y)
        hpelm = HPELM(1, 1)
        e = hpelm.error(fT, fY)
        np.testing.assert_allclose(e, err1)

    def test_ClassificationError_Works(self):
        T = self.makeh5(np.array([[0, 1], [0, 1], [1, 0]]))
        Y = self.makeh5(np.array([[0, 1], [0.4, 0.6], [0, 1]]))
        hpelm = HPELM(1, 2)
        hpelm.add_neurons(1, "lin")
        hpelm.classification = "c"
        e = hpelm.error(T, Y)
        np.testing.assert_allclose(e, 1.0 / 3)

    def test_WeightedClassError_Works(self):
        X = self.makeh5(np.array([1, 2, 3]))
        T = self.makeh5(np.array([[0, 1], [0, 1], [1, 0]]))
        Y = self.makeh5(np.array([[0, 1], [0.4, 0.6], [0, 1]]))
        # here class 0 is totally incorrect, and class 1 is totally correct
        w = (9, 1)
        hpelm = HPELM(1, 2)
        hpelm.add_neurons(1, "lin")
        hpelm.train(X, T, "wc", w=w)
        e = hpelm.error(T, Y)
        np.testing.assert_allclose(e, 0.9)

    def test_MultiLabelClassError_Works(self):
        T = self.makeh5(np.array([[0, 1], [1, 1], [1, 0]]))
        Y = self.makeh5(np.array([[0.4, 0.6], [0.8, 0.6], [1, 1]]))
        hpelm = HPELM(1, 2)
        hpelm.add_neurons(1, "lin")
        hpelm.classification = "ml"
        e = hpelm.error(T, Y)
        np.testing.assert_allclose(e, 1.0 / 6)






